//
//  TransactionViewController.swift
//  DemoPay
//
//  Created by Flex on 17.10.18.
//  Copyright © 2018 Flex. All rights reserved.
//

import Foundation


import UIKit

class TransactioniewController: EMVViewController{

    var fiscal: fpBase? = nil

    func hasPinpad() -> Bool {
        return (lib.getSupportedFeature(.FEAT_EMVL2_KERNEL, error: nil) & FEAT_EMV2_KERNELS.EMV_KERNEL_UNIVERSAL.rawValue) != 0
    }

    func hasESCPOSPrinter() -> Bool {
        return lib.getSupportedFeature(.FEAT_PRINTING, error: nil) != 0
    }

    func hasFiscalPrinter() -> Bool {
        return fiscal != nil && fiscal!.deviceConnected
    }

    func hasPrinter() -> Bool {
        return hasESCPOSPrinter() || hasFiscalPrinter()
    }

    public func fiscalConnect(inStream: InputStream, outStream: OutputStream) {
        fiscal = fpBase()
        do {
            try fiscal?.connect(withStreams: inStream, outputStream: outStream, packetFormat:PACKET_FORMAT.NEW)

            let info = try fiscal?.customCommand(90, data: "#")
            NSLog("Info: %@", info!)

        }catch {
            fiscalDisconnect()
        }
    }

    public func fiscalDisconnect() {
        fiscal?.disconnect()
        fiscal = nil
    }


    ///////////////////////////////////// IHUB

    func iHUBDeviceConnected(_ device: iHUBDevice!) {
        NSLog("iHUBDeviceConnected\nPort: %d\nConfig: %@",device.portIndex,device.portConfig)

        if device.portConfig != nil && device.portConfig!.starts(with: "PrinterFiscalNew") {
            if device.portType == .RS232 {
                let rsDevice = device as! iHUBRS232Device
                do {
                    try rsDevice.setRS232ConfigWithBaud(115200, dataBits: 8, stop: ._1, parity: .NONE, flowControl: .NONE)
                }catch {
                }
            }

            fiscalConnect(inStream: device.inputStream, outStream: device.outputStream)
        }
    }

    func iHUBDeviceDisconnected(_ device: iHUBDevice!) {
        NSLog("iHUBDeviceConnected\nPort: %d\nConfig: %@",device.portIndex,device.portConfig)
        if device.portConfig != nil && device.portConfig!.starts(with: "PrinterFiscalNew") {
            fiscal = nil
        }
    }

    func startTransaction(amount: Int) {

        if UIViewController.alert != nil {
            UIViewController.alert?.dismiss(animated: false, completion: nil)
            UIViewController.alert = nil
        }

        //stop barcode scanning if active
        do {
            try lib.barcodeStopScan()
        } catch {
        }

        if  (hasPinpad() && hasPrinter()) {
            UIViewController.alert = UIAlertController(title: "Payment method", message: "How do you want to pay?", preferredStyle: UIAlertControllerStyle.alert)

            let cashAction = UIAlertAction(title: "Cash", style: UIAlertActionStyle.default) {
                (result : UIAlertAction) -> Void in

                self.emvTransactionComplete(approved: true, receipt: "", card: nil)
            }
            let cardAction = UIAlertAction(title: "Card", style: UIAlertActionStyle.default) {
                (result : UIAlertAction) -> Void in
                _ = self.emvStartTransaction(amount: amount)
            }

            UIViewController.alert!.addAction(cashAction)
            UIViewController.alert!.addAction(cardAction)
            self.present(UIViewController.alert!, animated: true, completion: nil)
        }else {
            if hasPinpad() {
                _ = self.emvStartTransaction(amount: amount)
            }else {
                self.emvTransactionComplete(approved: true, receipt: "", card: nil)
            }
        }
    }

}

